/*	This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */

package com.nonoo.audiorecorder;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;

import javax.sound.sampled.AudioFileFormat;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.TargetDataLine;

public class AudioRecorder implements Runnable {

	private static final int SAMPLERATE = 8000; 
	private static final int BUFFERSIZE = SAMPLERATE*2; 
	
	private TargetDataLine tdl;
	private FileOutputStream fileOutputStream;

	AudioRecorder(final TargetDataLine tdl, final FileOutputStream fileOutputStream) {
		this.tdl = tdl;
		this.fileOutputStream = fileOutputStream;
	}

	@Override
	public void run() {
		byte[] abBuffer = new byte[BUFFERSIZE];

		ByteArrayOutputStream baos = new ByteArrayOutputStream(); // this will store sound data

		tdl.start();

		try {
			while (!Thread.interrupted()) {
				// waiting for the buffer to get filled
				while (tdl.available() < tdl.getBufferSize() * 0.5)
					Thread.sleep(0, 1); // without this, the audio will be choppy
	
				int bytesRead = tdl.read(abBuffer, 0, tdl.available());
				baos.write(abBuffer, 0, bytesRead);
				System.out.println("Captured " + bytesRead + " bytes.");
			}
		} catch (InterruptedException e) {
		}

		tdl.stop();
		tdl.close();

		// now writing the stored data into a .wav format file
		byte[] abData = baos.toByteArray();
		ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(abData);

		AudioInputStream audioInputStream = new AudioInputStream(byteArrayInputStream, tdl.getFormat(), abData.length / tdl.getFormat().getFrameSize());
		try {
			AudioSystem.write(audioInputStream, AudioFileFormat.Type.WAVE, fileOutputStream);
		} catch (IOException e) {
			e.printStackTrace();
		}

		try {
			audioInputStream.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public static void main(String[] args) {
		AudioFormat audioFormat = new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, SAMPLERATE, 16, 1, 2, SAMPLERATE, false);
		DataLine.Info info = new DataLine.Info(TargetDataLine.class, audioFormat, BUFFERSIZE);

		TargetDataLine targetDataLine = null;
		try {
			targetDataLine = (TargetDataLine) AudioSystem.getLine(info);
			targetDataLine.open(audioFormat, BUFFERSIZE);
			System.out.println("Buffer size: " + targetDataLine.getBufferSize());
		} catch (LineUnavailableException e1) {
			e1.printStackTrace();
		}

		FileOutputStream outputStream = null;
		try {
			outputStream = new FileOutputStream("output.wav");
		} catch (FileNotFoundException e1) {
			e1.printStackTrace();
		}

		// creating the recorder thread from this class' instance
		AudioRecorder audioRecorderDirect = new AudioRecorder(targetDataLine, outputStream);
		Thread audioRecorderThread = new Thread(audioRecorderDirect);

		// we use this to read line from the standard input
		BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
		System.out.println("Press ENTER to start recording!");
		try {
			br.readLine();
		} catch (IOException e) {
			e.printStackTrace();
		}

		audioRecorderThread.setPriority(Thread.MAX_PRIORITY);
		audioRecorderThread.start();

		System.out.println("Recording... press ENTER to stop recording!");
		try {
			br.readLine();
		} catch (IOException e) {
			e.printStackTrace();
		}

		audioRecorderThread.interrupt();

		try {
			// waiting for the recorder thread to stop
			audioRecorderThread.join();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		System.out.println("Recording stopped.");
	}
}
